/*********************************

    buffer I/O

**********************************/
#include    <stdio.h>
#include    <stdlib.h>
#include    <string.h>
#include    <ctype.h>
#include    <fcntl.h>
#include    <sys\types.h>
#include    <sys\stat.h>
#include    <io.h>
#include    "defs.h"

#define TMPFILE     999
#define	ocreat(f)   open(f,O_RDWR|O_CREAT|O_BINARY,S_IREAD|S_IWRITE)

/*****
#define BLK_SIZ         sizeof(BUF_PTR)
******/
#define BLK_SIZ         1024
#define BUF_SIZ         11
#define MAX_BLK         32

#define BUF_LIN(n)      ((unsigned)n*BUF_SIZ)
#define LIN_BUF(n)      ((unsigned)n/BUF_SIZ)
#define LIN_MASK(n)     ((unsigned)n%BUF_SIZ)

#define SEEK_POS(n)     ((long)(n)*BLK_SIZ)

typedef struct _BP {
    struct _BP  *next;
    int         alno;
    LIN_PTR     buf[BUF_SIZ];
} BUF_PTR;

static int      tfp=ERR;
static int      free_no=ERR;
static BUF_PTR  *top_buf=NULL;
static char     tmp_device[40];
static char     tmp_file_name[80];
static BUF_PTR  block_buff[MAX_BLK];

int     BUF_init(drv)
char    *drv;
{
    int     i;
    BUF_PTR *bp;

    i = MAX_BLK;
    top_buf = bp = block_buff;
    for ( i-- ; i > 0 ; i-- ) {
        bp->next = bp + 1;
        bp->alno = ERR;
        bp++;
    }
    bp->next = NULL;
    bp->alno = ERR;

    if ( drv == NULL )
        drv = ".";
    strcpy(tmp_device,drv);
    sprintf(tmp_file_name,"%s\\whistmp.%03d",tmp_device,TMPFILE);

    return FALSE;
}
void    BUF_end()
{
    if ( tfp != ERR ) {
        close(tfp);
        unlink(tmp_file_name);
    }
}
static void BUF_retry(no)
int     no;
{
    static char *msg[]={
        "Save Temp Seek Error        ",
        "Save Temp Write Header Error",
        "Save Temp Write Buffer Error",
        "Load Temp Seek Error        ",
        "Load Temp Read Header Error ",
        "Save Temp Read Buffer Error ",
        "New Temp File Can't open    ",
        "New Temp Write Error        ",
        "Temp File Can't open        "
    };
    int     i,rt=0;
    int     fp;
    char    *p;
    char    tmp[80];
    char    buf[512];

RETRY:

    do {
        sprintf(tmp,"%s: New Device ? ",msg[no]);
        while ( (p = input(tmp,tmp_device)) == NULL );
        strcpy(tmp_device,p);
        sprintf(tmp,"%s\\whistmp.%03d",tmp_device,TMPFILE);
    } while ( (fp = ocreat(tmp)) == ERR );

    if ( tfp != ERR ) {
        lseek(tfp,0L,0);
        while ( (i = read(tfp,buf,512)) > 0 ) {
            if ( write(fp,buf,i) < 0 )
                break;
        }
        if ( i < 0 ) {
            close(fp);
            unlink(tmp);
            no = 7;
            goto RETRY;
        }
        close(tfp);
        unlink(tmp_file_name);
    }

    tfp = fp;
    strcpy(tmp_file_name,tmp);
}
static void    BUF_save(bp)
register BUF_PTR *bp;
{
    if ( tfp == ERR && (tfp = ocreat(tmp_file_name)) == ERR ) {
        BUF_retry(8);
        goto RETRY;
    }
RETRY:
    if ( lseek(tfp,SEEK_POS(bp->alno),0) < 0 ) {
        BUF_retry(0);
        goto RETRY;
    }
    if ( write(tfp,(char *)bp,sizeof(BUF_PTR)) != sizeof(BUF_PTR) ) {
        BUF_retry(1);
        goto RETRY;
    }
}
static void BUF_load(no,bp)
int     no;
register BUF_PTR *bp;
{
    BUF_PTR *tp;

    if ( tfp == ERR && (tfp = ocreat(tmp_file_name)) == ERR ) {
        BUF_retry(8);
        goto RETRY;
    }
RETRY:
    if ( lseek(tfp,SEEK_POS(no),0) < 0 ) {
        BUF_retry(3);
        goto RETRY;
    }
    tp = bp->next;
    if ( read(tfp,(char *)bp,sizeof(BUF_PTR)) != sizeof(BUF_PTR) ) {
        bp->next = tp;
        BUF_retry(4);
        goto RETRY;
    }
    bp->next = tp;
}
static BUF_PTR *get_buf(no)
int     no;
{
    BUF_PTR *tp;
    register BUF_PTR *bp;

    bp = top_buf;
    while ( bp->alno != no ) {
        if ( bp->next == NULL ) {
            if ( bp->alno != ERR )
                BUF_save(bp);
            if ( no != ERR )
                BUF_load(no,bp);
            break;
        }
        tp = bp;
        bp = bp->next;
    }
    if ( bp != top_buf ) {
        tp->next = bp->next;
        bp->next = top_buf;
        top_buf = bp;
    }
    return bp;
}
LIN_PTR *get_lin(no)
int     no;
{
    register BUF_PTR *bp;

    bp = get_buf(LIN_BUF(no));
    return &(bp->buf[LIN_MASK(no)]);
}
int     xalloc()
{
    static int alno=0;

    int     i;
    unsigned n;
    LIN_PTR *lp;
    register BUF_PTR *bp;

    if ( free_no == ERR ) {
        bp = get_buf(ERR);
        bp->alno = alno++;
        n = BUF_LIN(bp->alno);
        for ( i = 0 ; i < BUF_SIZ ; i++,n++ ) {
            lp = &(bp->buf[i]);
            lp->left = free_no;
            lp->bp_no = n;
            free_no = n;
        }
    }
    n = free_no;
    lp = get_lin(free_no);
    free_no = lp->left;
    lp->left = lp->right = ERR;
    lp->lin[0] = '\0';
    lp->new_flg = TRUE;
    return n;
}
int     xfree(no)
int     no;
{
    register LIN_PTR *lp;

    lp = get_lin(no);
    lp->left = free_no;
    free_no = no;
}
