/*
	program	ldbeta6.c
*/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	"ioport.h"


#define		vram			0x0120
#define		phys			0x0060
#define		vram_acc		0x0458
#define		orgofs			     0
#define		xwbyte			   320

#define		video_out_ctrl	0x0448
#define		priority		  0x01

#define		alg_pal			0xfd90
#define		blue_pal		0xfd92
#define		red_pal			0xfd94
#define		green_pal		0xfd96

#define		rdiv(a,b)		(((a)+(b)/2)/(b))


typedef	struct{
			int		h,
					s,
					v;
}		hsvty;

typedef	struct{
			int		r,
					g,
					b;
}		rgbty;


char	*plane[4];


extern	void
		mkplane(
			char	*plane[]
		),
		rsplane(
			char	*plane[]
		);
extern	int
		ldpal(
			char	*name
		);
extern	void
		rdpal(
			FILE	*algfile,
			hsvty	pal[]
		);
extern	int
		getw(
			FILE	*stream
		);
extern	void
		torgb(
			hsvty	pal[],
			rgbty	pal2[]
		),
		setpal(
			rgbty	pal[]
		);


int
ldplane(
	char	*name,
	char	*ext,
	void	*buff
){
	int		rc;
	char	f[128];
	FILE	*fp;

	strcpy(f,name);
	strcat(f,ext);
	fp = fopen(f,"rb");
	if(fp!=NULL){
		fread(buff,32000,1,fp);
		fclose(fp);
		rc = 0;
	}else{
		rc = 1;
	}
	return(rc);
}


void
main(
	int		ac,
	char	**ag
){
	char	*name;

	if(ac==1){
		printf("usage : ldbeta6 <betafile> \n");
		exit(1);
	}else{
		name = ag[1];
	}

	mkplane(plane);
	ldplane(name,".b1",plane[0]);
	ldplane(name,".r1",plane[1]);
	ldplane(name,".g1",plane[2]);
	ldplane(name,".e1",plane[3]);
	rsplane(plane);
	ldpal(name);
}
		

void
mkplane(
	char	*plane[]
){
	char	*b1p,*r1p,*g1p,*e1p;
	int		bdat,
			b1d,r1d,g1d,e1d,
			i,j;
				
	plane[0] = malloc(32000);
	plane[1] = malloc(32000);
	plane[2] = malloc(32000);
	plane[3] = malloc(32000);
				
	b1p = plane[0];
	r1p = plane[1];
	g1p = plane[2];
	e1p = plane[3];
	
	for(i=0;i<128000;i+=4){
		bdat = peekdx(vram,i);

		b1d = 0;
		r1d = 0;
		g1d = 0;
		e1d = 0;
				
		for(j=0;j<8;++j){
			b1d    = b1d<<1 | bdat&1;
			bdat >>= 1;
			r1d    = r1d<<1 | bdat&1;
			bdat >>= 1;
			g1d    = g1d<<1 | bdat&1;
			bdat >>= 1;
			e1d    = e1d<<1 | bdat&1;
			bdat >>= 1;
		}

		*(b1p++) = b1d;
		*(r1p++) = r1d;
		*(g1p++) = g1d;
		*(e1p++) = e1d;
	}
}


void
rsplane(
	char	*plane[]
){
	int		i,j,
			vptr;
	char	*b1p,*r1p,*g1p,*e1p;

	outregw(vram_acc,0,0xffff);
	outregw(vram_acc,1,0xffff);

	b1p  = plane[0];
	r1p  = plane[1];
	g1p  = plane[2];
	e1p  = plane[3];

	vptr = orgofs;
	for(j=0;j<400;++j){
		for(i=0;i<80;++i){
			pokedx(vram,vptr,	 tob16c[*e1p]<<3
								|tob16c[*g1p]<<2
								|tob16c[*r1p]<<1
								|tob16c[*b1p]   );
			++b1p;
			++r1p;
			++g1p;
			++e1p;
			vptr += 4;
		}
		vptr += xwbyte-320;
	}

	free(plane[0]);
	free(plane[1]);
	free(plane[2]);
	free(plane[3]);
}


int
ldpal(
	char	*name
){
	int		rc;
	FILE	*af;
	char	path[80];
	hsvty	palx[16];
	rgbty	pal [16];

	enable16c();

	rc = 0;
	strcpy(path,name);
	strcat(path,".alg");
	af = fopen(path,"rb");
	if(af!=NULL){
		rdpal (af,palx);
		fclose(af);
		torgb(palx,pal);
		setpal(pal);
	}else{
		rc = 1;
	}
	return(rc);
}


void
rdpal(
	FILE	*algfile,
	hsvty	pal[]
){
	int		i;

	pal[0].h = getw(algfile);
	for(i=9;i<=15;++i){
		pal[i].h = getw(algfile);
	}
	pal[0].s = getw(algfile);
	for(i=9;i<=15;++i){
		pal[i].s = getw(algfile);
	}
	pal[0].v = getw(algfile);
	for(i=9;i<=15;++i){
		pal[i].v = getw(algfile);
	}

	pal[8].h = getw(algfile);
	for(i=1;i<=7;++i){
		pal[i].h = getw(algfile);
	}
	pal[8].s = getw(algfile);
	for(i=1;i<=7;++i){
		pal[i].s = getw(algfile);
	}
	pal[8].v = getw(algfile);
	for(i=1;i<=7;++i){
		pal[i].v = getw(algfile);
	}
}


int
getw(
	FILE	*stream
){
	int		wl,wh;
	
	wl = getc(stream);
	wh = getc(stream);

	return((wh<<8)+wl);
}
	


void
torgb(
	hsvty	pal[],
	rgbty	pal2[]
){
	int		imax = 15;
	int		p;
	long	h,s,v,i,f,m,n,k,r,g,b;

	for(p=0;p<16;++p){
		h = pal[p].h;			/* 1/360 */
		s = pal[p].s;			/* 1/100 */
		v = pal[p].v;			/* 1/100 */

		i = h/60;
		f = h-60*i;				/* 1/60 */
		m = v*(100-s);			/* 1/10000 */
		n = v*(6000-s*f);		/* 1/600000 */
		k = v*(6000-s*(60-f));	/* 1/600000 */

		switch((int)i){
			case 0:
				r = rdiv(imax*v,   100L);
				g = rdiv(imax*k,600000L);
				b = rdiv(imax*m, 10000L);
				break;
			case 1:
				r = rdiv(imax*n,600000L);
				g = rdiv(imax*v,   100L);
				b = rdiv(imax*m, 10000L);
				break;
			case 2:
				r = rdiv(imax*m, 10000L);
				g = rdiv(imax*v,   100L);
				b = rdiv(imax*k,600000L);
				break;
			case 3:
				r = rdiv(imax*m, 10000L);
				g = rdiv(imax*n,600000L);
				b = rdiv(imax*v,   100L);
				break;
			case 4:
				r = rdiv(imax*k,600000L);
				g = rdiv(imax*m, 10000L);
				b = rdiv(imax*v,   100L);
				break;
			case 5:
				r = rdiv(imax*v,   100L);
				g = rdiv(imax*m, 10000L);
				b = rdiv(imax*n,600000L);
				break;
		}

		pal2[p].r = (int)r;
		pal2[p].g = (int)g;
		pal2[p].b = (int)b;
	}
}


void
setpal(
	rgbty	pal[]
){
	int		i;

	outregb(video_out_ctrl,priority,1|0x08);
										/* layer 0,16c pal /  adv layer 1 */
	for(i=0;i<16;++i){
		outpb(alg_pal	,i);
		outpb(blue_pal	,pal[i].b<<4);
		outpb(red_pal	,pal[i].r<<4);
		outpb(green_pal	,pal[i].g<<4);
	}
}

		
/*
	end of prog
*/
